use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
shantanu
bhadoria
CBusDevicePath
CDeviceAddress
Datasheet
dev
libi
smbus
xs
processCall
readByte
readByteData
readNBytes
readBlockData
readWordData
writeByte
writeByteData
writeQuick
writeWordData
Destructor
fileError
conf
opkg
ArchLINUX
att
doesnt
raspbian
writeBlockData
ASF
Accelerometer
ESC
IPMI
PCI
PMBus
PWM
Quadrotor
AccelerationVectorAngles
AccelerationVectorInG
AltIMU
Beaglebone
DLHC
IMU
LPS
LSM
accelerometer
basiclly
chipset
chipsets
gravitationally
onboard
pololu
Shantanu
Bhadoria
https
Jonathan
Stowe
jns
Neil
Bowers
neil
cpan
dott
org
wfreller
wolfgang
lib
Device
SMBus
Manual
